# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tests import (
    TestsResource,
    AsyncTestsResource,
    TestsResourceWithRawResponse,
    AsyncTestsResourceWithRawResponse,
    TestsResourceWithStreamingResponse,
    AsyncTestsResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from .assistants import (
    AssistantsResource,
    AsyncAssistantsResource,
    AssistantsResourceWithRawResponse,
    AsyncAssistantsResourceWithRawResponse,
    AssistantsResourceWithStreamingResponse,
    AsyncAssistantsResourceWithStreamingResponse,
)
from .canary_deploys import (
    CanaryDeploysResource,
    AsyncCanaryDeploysResource,
    CanaryDeploysResourceWithRawResponse,
    AsyncCanaryDeploysResourceWithRawResponse,
    CanaryDeploysResourceWithStreamingResponse,
    AsyncCanaryDeploysResourceWithStreamingResponse,
)
from .scheduled_events import (
    ScheduledEventsResource,
    AsyncScheduledEventsResource,
    ScheduledEventsResourceWithRawResponse,
    AsyncScheduledEventsResourceWithRawResponse,
    ScheduledEventsResourceWithStreamingResponse,
    AsyncScheduledEventsResourceWithStreamingResponse,
)

__all__ = [
    "TestsResource",
    "AsyncTestsResource",
    "TestsResourceWithRawResponse",
    "AsyncTestsResourceWithRawResponse",
    "TestsResourceWithStreamingResponse",
    "AsyncTestsResourceWithStreamingResponse",
    "CanaryDeploysResource",
    "AsyncCanaryDeploysResource",
    "CanaryDeploysResourceWithRawResponse",
    "AsyncCanaryDeploysResourceWithRawResponse",
    "CanaryDeploysResourceWithStreamingResponse",
    "AsyncCanaryDeploysResourceWithStreamingResponse",
    "ScheduledEventsResource",
    "AsyncScheduledEventsResource",
    "ScheduledEventsResourceWithRawResponse",
    "AsyncScheduledEventsResourceWithRawResponse",
    "ScheduledEventsResourceWithStreamingResponse",
    "AsyncScheduledEventsResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "AssistantsResource",
    "AsyncAssistantsResource",
    "AssistantsResourceWithRawResponse",
    "AsyncAssistantsResourceWithRawResponse",
    "AssistantsResourceWithStreamingResponse",
    "AsyncAssistantsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import oauth_client_list_params, oauth_client_create_params, oauth_client_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.oauth_client_list_response import OAuthClientListResponse
from ..types.oauth_client_create_response import OAuthClientCreateResponse
from ..types.oauth_client_update_response import OAuthClientUpdateResponse
from ..types.oauth_client_retrieve_response import OAuthClientRetrieveResponse

__all__ = ["OAuthClientsResource", "AsyncOAuthClientsResource"]


class OAuthClientsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OAuthClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OAuthClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OAuthClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OAuthClientsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        allowed_grant_types: List[Literal["client_credentials", "authorization_code", "refresh_token"]],
        allowed_scopes: SequenceNotStr[str],
        client_type: Literal["public", "confidential"],
        name: str,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        require_pkce: bool | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientCreateResponse:
        """
        Create a new OAuth client

        Args:
          allowed_grant_types: List of allowed OAuth grant types

          allowed_scopes: List of allowed OAuth scopes

          client_type: OAuth client type

          name: The name of the OAuth client

          logo_uri: URL of the client logo

          policy_uri: URL of the client's privacy policy

          redirect_uris: List of redirect URIs (required for authorization_code flow)

          require_pkce: Whether PKCE (Proof Key for Code Exchange) is required for this client

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth_clients",
            body=maybe_transform(
                {
                    "allowed_grant_types": allowed_grant_types,
                    "allowed_scopes": allowed_scopes,
                    "client_type": client_type,
                    "name": name,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "require_pkce": require_pkce,
                    "tos_uri": tos_uri,
                },
                oauth_client_create_params.OAuthClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientRetrieveResponse:
        """
        Retrieve a single OAuth client by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/oauth_clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        allowed_grant_types: List[Literal["client_credentials", "authorization_code", "refresh_token"]] | Omit = omit,
        allowed_scopes: SequenceNotStr[str] | Omit = omit,
        logo_uri: str | Omit = omit,
        name: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        require_pkce: bool | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientUpdateResponse:
        """
        Update an existing OAuth client

        Args:
          allowed_grant_types: List of allowed OAuth grant types

          allowed_scopes: List of allowed OAuth scopes

          logo_uri: URL of the client logo

          name: The name of the OAuth client

          policy_uri: URL of the client's privacy policy

          redirect_uris: List of redirect URIs

          require_pkce: Whether PKCE (Proof Key for Code Exchange) is required for this client

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/oauth_clients/{id}",
            body=maybe_transform(
                {
                    "allowed_grant_types": allowed_grant_types,
                    "allowed_scopes": allowed_scopes,
                    "logo_uri": logo_uri,
                    "name": name,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "require_pkce": require_pkce,
                    "tos_uri": tos_uri,
                },
                oauth_client_update_params.OAuthClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientUpdateResponse,
        )

    def list(
        self,
        *,
        filter_allowed_grant_types_contains: Literal["client_credentials", "authorization_code", "refresh_token"]
        | Omit = omit,
        filter_client_id: str | Omit = omit,
        filter_client_type: Literal["confidential", "public"] | Omit = omit,
        filter_name: str | Omit = omit,
        filter_name_contains: str | Omit = omit,
        filter_verified: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientListResponse:
        """
        Retrieve a paginated list of OAuth clients for the authenticated user

        Args:
          filter_allowed_grant_types_contains: Filter by allowed grant type

          filter_client_id: Filter by client ID

          filter_client_type: Filter by client type

          filter_name: Filter by exact client name

          filter_name_contains: Filter by client name containing text

          filter_verified: Filter by verification status

          page_number: Page number

          page_size: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/oauth_clients",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_allowed_grant_types_contains": filter_allowed_grant_types_contains,
                        "filter_client_id": filter_client_id,
                        "filter_client_type": filter_client_type,
                        "filter_name": filter_name,
                        "filter_name_contains": filter_name_contains,
                        "filter_verified": filter_verified,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    oauth_client_list_params.OAuthClientListParams,
                ),
            ),
            cast_to=OAuthClientListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an OAuth client

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/oauth_clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncOAuthClientsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOAuthClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOAuthClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOAuthClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOAuthClientsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        allowed_grant_types: List[Literal["client_credentials", "authorization_code", "refresh_token"]],
        allowed_scopes: SequenceNotStr[str],
        client_type: Literal["public", "confidential"],
        name: str,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        require_pkce: bool | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientCreateResponse:
        """
        Create a new OAuth client

        Args:
          allowed_grant_types: List of allowed OAuth grant types

          allowed_scopes: List of allowed OAuth scopes

          client_type: OAuth client type

          name: The name of the OAuth client

          logo_uri: URL of the client logo

          policy_uri: URL of the client's privacy policy

          redirect_uris: List of redirect URIs (required for authorization_code flow)

          require_pkce: Whether PKCE (Proof Key for Code Exchange) is required for this client

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth_clients",
            body=await async_maybe_transform(
                {
                    "allowed_grant_types": allowed_grant_types,
                    "allowed_scopes": allowed_scopes,
                    "client_type": client_type,
                    "name": name,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "require_pkce": require_pkce,
                    "tos_uri": tos_uri,
                },
                oauth_client_create_params.OAuthClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientRetrieveResponse:
        """
        Retrieve a single OAuth client by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/oauth_clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        allowed_grant_types: List[Literal["client_credentials", "authorization_code", "refresh_token"]] | Omit = omit,
        allowed_scopes: SequenceNotStr[str] | Omit = omit,
        logo_uri: str | Omit = omit,
        name: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        require_pkce: bool | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientUpdateResponse:
        """
        Update an existing OAuth client

        Args:
          allowed_grant_types: List of allowed OAuth grant types

          allowed_scopes: List of allowed OAuth scopes

          logo_uri: URL of the client logo

          name: The name of the OAuth client

          policy_uri: URL of the client's privacy policy

          redirect_uris: List of redirect URIs

          require_pkce: Whether PKCE (Proof Key for Code Exchange) is required for this client

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/oauth_clients/{id}",
            body=await async_maybe_transform(
                {
                    "allowed_grant_types": allowed_grant_types,
                    "allowed_scopes": allowed_scopes,
                    "logo_uri": logo_uri,
                    "name": name,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "require_pkce": require_pkce,
                    "tos_uri": tos_uri,
                },
                oauth_client_update_params.OAuthClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthClientUpdateResponse,
        )

    async def list(
        self,
        *,
        filter_allowed_grant_types_contains: Literal["client_credentials", "authorization_code", "refresh_token"]
        | Omit = omit,
        filter_client_id: str | Omit = omit,
        filter_client_type: Literal["confidential", "public"] | Omit = omit,
        filter_name: str | Omit = omit,
        filter_name_contains: str | Omit = omit,
        filter_verified: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthClientListResponse:
        """
        Retrieve a paginated list of OAuth clients for the authenticated user

        Args:
          filter_allowed_grant_types_contains: Filter by allowed grant type

          filter_client_id: Filter by client ID

          filter_client_type: Filter by client type

          filter_name: Filter by exact client name

          filter_name_contains: Filter by client name containing text

          filter_verified: Filter by verification status

          page_number: Page number

          page_size: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/oauth_clients",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter_allowed_grant_types_contains": filter_allowed_grant_types_contains,
                        "filter_client_id": filter_client_id,
                        "filter_client_type": filter_client_type,
                        "filter_name": filter_name,
                        "filter_name_contains": filter_name_contains,
                        "filter_verified": filter_verified,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    oauth_client_list_params.OAuthClientListParams,
                ),
            ),
            cast_to=OAuthClientListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an OAuth client

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/oauth_clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class OAuthClientsResourceWithRawResponse:
    def __init__(self, oauth_clients: OAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = to_raw_response_wrapper(
            oauth_clients.create,
        )
        self.retrieve = to_raw_response_wrapper(
            oauth_clients.retrieve,
        )
        self.update = to_raw_response_wrapper(
            oauth_clients.update,
        )
        self.list = to_raw_response_wrapper(
            oauth_clients.list,
        )
        self.delete = to_raw_response_wrapper(
            oauth_clients.delete,
        )


class AsyncOAuthClientsResourceWithRawResponse:
    def __init__(self, oauth_clients: AsyncOAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = async_to_raw_response_wrapper(
            oauth_clients.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            oauth_clients.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            oauth_clients.update,
        )
        self.list = async_to_raw_response_wrapper(
            oauth_clients.list,
        )
        self.delete = async_to_raw_response_wrapper(
            oauth_clients.delete,
        )


class OAuthClientsResourceWithStreamingResponse:
    def __init__(self, oauth_clients: OAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = to_streamed_response_wrapper(
            oauth_clients.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            oauth_clients.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            oauth_clients.update,
        )
        self.list = to_streamed_response_wrapper(
            oauth_clients.list,
        )
        self.delete = to_streamed_response_wrapper(
            oauth_clients.delete,
        )


class AsyncOAuthClientsResourceWithStreamingResponse:
    def __init__(self, oauth_clients: AsyncOAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = async_to_streamed_response_wrapper(
            oauth_clients.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            oauth_clients.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            oauth_clients.update,
        )
        self.list = async_to_streamed_response_wrapper(
            oauth_clients.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            oauth_clients.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["IntegrationSecretListParams", "Filter", "Page"]


class IntegrationSecretListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style). Originally: filter[type]"""

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class Filter(TypedDict, total=False):
    type: Literal["bearer", "basic"]


class Page(TypedDict, total=False):
    number: int

    size: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["PortingOrderDocuments"]


class PortingOrderDocuments(BaseModel):
    invoice: Optional[str] = None
    """Returned ID of the submitted Invoice via the Documents endpoint"""

    loa: Optional[str] = None
    """Returned ID of the submitted LOA via the Documents endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["SimCardListWirelessConnectivityLogsResponse", "Data"]


class Data(BaseModel):
    id: Optional[int] = None
    """Uniquely identifies the session."""

    apn: Optional[str] = None
    """
    The Access Point Name (APN) identifies the packet data network that a mobile
    data user wants to communicate with.
    """

    cell_id: Optional[str] = None
    """The cell ID to which the SIM connected."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the record was created."""

    imei: Optional[str] = None
    """
    The International Mobile Equipment Identity (or IMEI) is a number, usually
    unique, that identifies the device currently being used connect to the network.
    """

    imsi: Optional[str] = None
    """
    SIM cards are identified on their individual network operators by a unique
    International Mobile Subscriber Identity (IMSI). <br/> Mobile network operators
    connect mobile phone calls and communicate with their market SIM cards using
    their IMSIs. The IMSI is stored in the Subscriber Identity Module (SIM) inside
    the device and is sent by the device to the appropriate network. It is used to
    acquire the details of the device in the Home Location Register (HLR) or the
    Visitor Location Register (VLR).
    """

    ipv4: Optional[str] = None
    """The SIM's address in the currently connected network.

    This IPv4 address is usually obtained dynamically, so it may vary according to
    the location or new connections.
    """

    ipv6: Optional[str] = None
    """The SIM's address in the currently connected network.

    This IPv6 address is usually obtained dynamically, so it may vary according to
    the location or new connections.
    """

    last_seen: Optional[str] = None
    """
    ISO 8601 formatted date-time indicating when the last heartbeat to the device
    was successfully recorded.
    """

    log_type: Optional[Literal["registration", "data"]] = None
    """
    The type of the session, 'registration' being the initial authentication session
    and 'data' the actual data transfer sessions.
    """

    mobile_country_code: Optional[str] = None
    """
    It's a three decimal digit that identifies a country.<br/><br/> This code is
    commonly seen joined with a Mobile Network Code (MNC) in a tuple that allows
    identifying a carrier known as PLMN (Public Land Mobile Network) code.
    """

    mobile_network_code: Optional[str] = None
    """
    It's a two to three decimal digits that identify a network.<br/><br/> This code
    is commonly seen joined with a Mobile Country Code (MCC) in a tuple that allows
    identifying a carrier known as PLMN (Public Land Mobile Network) code.
    """

    radio_access_technology: Optional[str] = None
    """The radio technology the SIM card used during the session."""

    record_type: Optional[str] = None

    sim_card_id: Optional[str] = None
    """The identification UUID of the related SIM card resource."""

    start_time: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the session started."""

    state: Optional[str] = None
    """The state of the SIM card after when the session happened."""

    stop_time: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the session ended."""


class SimCardListWirelessConnectivityLogsResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CommentListParams", "Filter"]


class CommentListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[comment_record_type], filter[comment_record_id]
    """


class Filter(TypedDict, total=False):
    comment_record_id: str
    """ID of the record the comments relate to"""

    comment_record_type: Literal["sub_number_order", "requirement_group"]
    """Record type that the comment relates to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from .conferences import (
    ConferencesResource,
    AsyncConferencesResource,
    ConferencesResourceWithRawResponse,
    AsyncConferencesResourceWithRawResponse,
    ConferencesResourceWithStreamingResponse,
    AsyncConferencesResourceWithStreamingResponse,
)
from .transcriptions import (
    TranscriptionsResource,
    AsyncTranscriptionsResource,
    TranscriptionsResourceWithRawResponse,
    AsyncTranscriptionsResourceWithRawResponse,
    TranscriptionsResourceWithStreamingResponse,
    AsyncTranscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
    "ConferencesResource",
    "AsyncConferencesResource",
    "ConferencesResourceWithRawResponse",
    "AsyncConferencesResourceWithRawResponse",
    "ConferencesResourceWithStreamingResponse",
    "AsyncConferencesResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
    "TranscriptionsResource",
    "AsyncTranscriptionsResource",
    "TranscriptionsResourceWithRawResponse",
    "AsyncTranscriptionsResourceWithRawResponse",
    "TranscriptionsResourceWithStreamingResponse",
    "AsyncTranscriptionsResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.ai.assistants import TelnyxConversationChannel, test_list_params, test_create_params, test_update_params
from .test_suites.test_suites import (
    TestSuitesResource,
    AsyncTestSuitesResource,
    TestSuitesResourceWithRawResponse,
    AsyncTestSuitesResourceWithRawResponse,
    TestSuitesResourceWithStreamingResponse,
    AsyncTestSuitesResourceWithStreamingResponse,
)
from .....types.ai.assistants.assistant_test import AssistantTest
from .....types.ai.assistants.test_list_response import TestListResponse
from .....types.ai.assistants.telnyx_conversation_channel import TelnyxConversationChannel

__all__ = ["TestsResource", "AsyncTestsResource"]


class TestsResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def test_suites(self) -> TestSuitesResource:
        return TestSuitesResource(self._client)

    @cached_property
    def runs(self) -> RunsResource:
        return RunsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TestsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        destination: str,
        instructions: str,
        name: str,
        rubric: Iterable[test_create_params.Rubric],
        description: str | Omit = omit,
        max_duration_seconds: int | Omit = omit,
        telnyx_conversation_channel: TelnyxConversationChannel | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Creates a comprehensive test configuration for evaluating AI assistant
        performance

        Args:
          destination:
              The target destination for the test conversation. Format depends on the channel:
              phone number for SMS/voice, webhook URL for web chat, etc.

          instructions: Detailed instructions that define the test scenario and what the assistant
              should accomplish. This guides the test execution and evaluation.

          name: A descriptive name for the assistant test. This will be used to identify the
              test in the UI and reports.

          rubric: Evaluation criteria used to assess the assistant's performance. Each rubric item
              contains a name and specific criteria for evaluation.

          description: Optional detailed description of what this test evaluates and its purpose. Helps
              team members understand the test's objectives.

          max_duration_seconds: Maximum duration in seconds that the test conversation should run before timing
              out. If not specified, uses system default timeout.

          telnyx_conversation_channel: The communication channel through which the test will be conducted. Determines
              how the assistant will receive and respond to test messages.

          test_suite: Optional test suite name to group related tests together. Useful for organizing
              tests by feature, team, or release cycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/assistants/tests",
            body=maybe_transform(
                {
                    "destination": destination,
                    "instructions": instructions,
                    "name": name,
                    "rubric": rubric,
                    "description": description,
                    "max_duration_seconds": max_duration_seconds,
                    "telnyx_conversation_channel": telnyx_conversation_channel,
                    "test_suite": test_suite,
                },
                test_create_params.TestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    def retrieve(
        self,
        test_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Retrieves detailed information about a specific assistant test

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._get(
            f"/ai/assistants/tests/{test_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    def update(
        self,
        test_id: str,
        *,
        description: str | Omit = omit,
        destination: str | Omit = omit,
        instructions: str | Omit = omit,
        max_duration_seconds: int | Omit = omit,
        name: str | Omit = omit,
        rubric: Iterable[test_update_params.Rubric] | Omit = omit,
        telnyx_conversation_channel: TelnyxConversationChannel | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Updates an existing assistant test configuration with new settings

        Args:
          description: Updated description of the test's purpose and evaluation criteria.

          destination: Updated target destination for test conversations.

          instructions: Updated test scenario instructions and objectives.

          max_duration_seconds: Updated maximum test duration in seconds.

          name: Updated name for the assistant test. Must be unique and descriptive.

          rubric: Updated evaluation criteria for assessing assistant performance.

          telnyx_conversation_channel: Updated communication channel for the test execution.

          test_suite: Updated test suite assignment for better organization.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._put(
            f"/ai/assistants/tests/{test_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "destination": destination,
                    "instructions": instructions,
                    "max_duration_seconds": max_duration_seconds,
                    "name": name,
                    "rubric": rubric,
                    "telnyx_conversation_channel": telnyx_conversation_channel,
                    "test_suite": test_suite,
                },
                test_update_params.TestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    def list(
        self,
        *,
        destination: str | Omit = omit,
        page: test_list_params.Page | Omit = omit,
        telnyx_conversation_channel: str | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestListResponse:
        """
        Retrieves a paginated list of assistant tests with optional filtering
        capabilities

        Args:
          destination: Filter tests by destination (phone number, webhook URL, etc.)

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          telnyx_conversation_channel: Filter tests by communication channel (e.g., 'web_chat', 'sms')

          test_suite: Filter tests by test suite name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/ai/assistants/tests",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "destination": destination,
                        "page": page,
                        "telnyx_conversation_channel": telnyx_conversation_channel,
                        "test_suite": test_suite,
                    },
                    test_list_params.TestListParams,
                ),
            ),
            cast_to=TestListResponse,
        )

    def delete(
        self,
        test_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Permanently removes an assistant test and all associated data

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._delete(
            f"/ai/assistants/tests/{test_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncTestsResource(AsyncAPIResource):
    @cached_property
    def test_suites(self) -> AsyncTestSuitesResource:
        return AsyncTestSuitesResource(self._client)

    @cached_property
    def runs(self) -> AsyncRunsResource:
        return AsyncRunsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTestsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        destination: str,
        instructions: str,
        name: str,
        rubric: Iterable[test_create_params.Rubric],
        description: str | Omit = omit,
        max_duration_seconds: int | Omit = omit,
        telnyx_conversation_channel: TelnyxConversationChannel | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Creates a comprehensive test configuration for evaluating AI assistant
        performance

        Args:
          destination:
              The target destination for the test conversation. Format depends on the channel:
              phone number for SMS/voice, webhook URL for web chat, etc.

          instructions: Detailed instructions that define the test scenario and what the assistant
              should accomplish. This guides the test execution and evaluation.

          name: A descriptive name for the assistant test. This will be used to identify the
              test in the UI and reports.

          rubric: Evaluation criteria used to assess the assistant's performance. Each rubric item
              contains a name and specific criteria for evaluation.

          description: Optional detailed description of what this test evaluates and its purpose. Helps
              team members understand the test's objectives.

          max_duration_seconds: Maximum duration in seconds that the test conversation should run before timing
              out. If not specified, uses system default timeout.

          telnyx_conversation_channel: The communication channel through which the test will be conducted. Determines
              how the assistant will receive and respond to test messages.

          test_suite: Optional test suite name to group related tests together. Useful for organizing
              tests by feature, team, or release cycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/assistants/tests",
            body=await async_maybe_transform(
                {
                    "destination": destination,
                    "instructions": instructions,
                    "name": name,
                    "rubric": rubric,
                    "description": description,
                    "max_duration_seconds": max_duration_seconds,
                    "telnyx_conversation_channel": telnyx_conversation_channel,
                    "test_suite": test_suite,
                },
                test_create_params.TestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    async def retrieve(
        self,
        test_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Retrieves detailed information about a specific assistant test

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._get(
            f"/ai/assistants/tests/{test_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    async def update(
        self,
        test_id: str,
        *,
        description: str | Omit = omit,
        destination: str | Omit = omit,
        instructions: str | Omit = omit,
        max_duration_seconds: int | Omit = omit,
        name: str | Omit = omit,
        rubric: Iterable[test_update_params.Rubric] | Omit = omit,
        telnyx_conversation_channel: TelnyxConversationChannel | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantTest:
        """
        Updates an existing assistant test configuration with new settings

        Args:
          description: Updated description of the test's purpose and evaluation criteria.

          destination: Updated target destination for test conversations.

          instructions: Updated test scenario instructions and objectives.

          max_duration_seconds: Updated maximum test duration in seconds.

          name: Updated name for the assistant test. Must be unique and descriptive.

          rubric: Updated evaluation criteria for assessing assistant performance.

          telnyx_conversation_channel: Updated communication channel for the test execution.

          test_suite: Updated test suite assignment for better organization.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._put(
            f"/ai/assistants/tests/{test_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "destination": destination,
                    "instructions": instructions,
                    "max_duration_seconds": max_duration_seconds,
                    "name": name,
                    "rubric": rubric,
                    "telnyx_conversation_channel": telnyx_conversation_channel,
                    "test_suite": test_suite,
                },
                test_update_params.TestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantTest,
        )

    async def list(
        self,
        *,
        destination: str | Omit = omit,
        page: test_list_params.Page | Omit = omit,
        telnyx_conversation_channel: str | Omit = omit,
        test_suite: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestListResponse:
        """
        Retrieves a paginated list of assistant tests with optional filtering
        capabilities

        Args:
          destination: Filter tests by destination (phone number, webhook URL, etc.)

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          telnyx_conversation_channel: Filter tests by communication channel (e.g., 'web_chat', 'sms')

          test_suite: Filter tests by test suite name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/ai/assistants/tests",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "destination": destination,
                        "page": page,
                        "telnyx_conversation_channel": telnyx_conversation_channel,
                        "test_suite": test_suite,
                    },
                    test_list_params.TestListParams,
                ),
            ),
            cast_to=TestListResponse,
        )

    async def delete(
        self,
        test_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Permanently removes an assistant test and all associated data

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._delete(
            f"/ai/assistants/tests/{test_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class TestsResourceWithRawResponse:
    __test__ = False

    def __init__(self, tests: TestsResource) -> None:
        self._tests = tests

        self.create = to_raw_response_wrapper(
            tests.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tests.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tests.update,
        )
        self.list = to_raw_response_wrapper(
            tests.list,
        )
        self.delete = to_raw_response_wrapper(
            tests.delete,
        )

    @cached_property
    def test_suites(self) -> TestSuitesResourceWithRawResponse:
        return TestSuitesResourceWithRawResponse(self._tests.test_suites)

    @cached_property
    def runs(self) -> RunsResourceWithRawResponse:
        return RunsResourceWithRawResponse(self._tests.runs)


class AsyncTestsResourceWithRawResponse:
    def __init__(self, tests: AsyncTestsResource) -> None:
        self._tests = tests

        self.create = async_to_raw_response_wrapper(
            tests.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tests.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tests.update,
        )
        self.list = async_to_raw_response_wrapper(
            tests.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tests.delete,
        )

    @cached_property
    def test_suites(self) -> AsyncTestSuitesResourceWithRawResponse:
        return AsyncTestSuitesResourceWithRawResponse(self._tests.test_suites)

    @cached_property
    def runs(self) -> AsyncRunsResourceWithRawResponse:
        return AsyncRunsResourceWithRawResponse(self._tests.runs)


class TestsResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, tests: TestsResource) -> None:
        self._tests = tests

        self.create = to_streamed_response_wrapper(
            tests.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tests.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tests.update,
        )
        self.list = to_streamed_response_wrapper(
            tests.list,
        )
        self.delete = to_streamed_response_wrapper(
            tests.delete,
        )

    @cached_property
    def test_suites(self) -> TestSuitesResourceWithStreamingResponse:
        return TestSuitesResourceWithStreamingResponse(self._tests.test_suites)

    @cached_property
    def runs(self) -> RunsResourceWithStreamingResponse:
        return RunsResourceWithStreamingResponse(self._tests.runs)


class AsyncTestsResourceWithStreamingResponse:
    def __init__(self, tests: AsyncTestsResource) -> None:
        self._tests = tests

        self.create = async_to_streamed_response_wrapper(
            tests.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tests.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tests.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tests.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tests.delete,
        )

    @cached_property
    def test_suites(self) -> AsyncTestSuitesResourceWithStreamingResponse:
        return AsyncTestSuitesResourceWithStreamingResponse(self._tests.test_suites)

    @cached_property
    def runs(self) -> AsyncRunsResourceWithStreamingResponse:
        return AsyncRunsResourceWithStreamingResponse(self._tests.runs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .telnyx_brand import TelnyxBrand

__all__ = ["BrandRetrieveResponse"]


class BrandRetrieveResponse(TelnyxBrand):
    assigned_campaigns_count: Optional[float] = FieldInfo(alias="assignedCampaignsCount", default=None)
    """Number of campaigns associated with the brand"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..room_session import RoomSession
from ..pagination_meta import PaginationMeta

__all__ = ["SessionList1Response"]


class SessionList1Response(BaseModel):
    data: Optional[List[RoomSession]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "DetailRecordListResponse",
    "Data",
    "DataMessageDetailRecord",
    "DataConferenceDetailRecord",
    "DataConferenceParticipantDetailRecord",
    "DataAmdDetailRecord",
    "DataVerifyDetailRecord",
    "DataSimCardUsageDetailRecord",
    "DataMediaStorageDetailRecord",
    "Meta",
]


class DataMessageDetailRecord(BaseModel):
    record_type: str
    """Identifies the record schema"""

    carrier: Optional[str] = None
    """Country-specific carrier used to send or receive the message"""

    carrier_fee: Optional[str] = None
    """Fee charged by certain carriers in order to deliver certain message types.

    Telnyx passes this fee on to the customer according to our pricing table
    """

    cld: Optional[str] = None
    """The recipient of the message (to parameter in the Messaging API)"""

    cli: Optional[str] = None
    """The sender of the message (from parameter in the Messaging API).

    For Alphanumeric ID messages, this is the sender ID value
    """

    completed_at: Optional[datetime] = None
    """Message completion time"""

    cost: Optional[str] = None
    """Amount, in the user currency, for the Telnyx billing cost"""

    country_code: Optional[str] = None
    """
    Two-letter representation of the country of the cld property using the ISO
    3166-1 alpha-2 format
    """

    created_at: Optional[datetime] = None
    """Message creation time"""

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing cost
    """

    delivery_status: Optional[str] = None
    """Final webhook delivery status"""

    delivery_status_failover_url: Optional[str] = None
    """Failover customer-provided URL which Telnyx posts delivery status webhooks to"""

    delivery_status_webhook_url: Optional[str] = None
    """Primary customer-provided URL which Telnyx posts delivery status webhooks to"""

    direction: Optional[Literal["inbound", "outbound"]] = None
    """Logical direction of the message from the Telnyx customer's perspective.

    It's inbound when the Telnyx customer receives the message, or outbound
    otherwise
    """

    errors: Optional[List[str]] = None
    """Telnyx API error codes returned by the Telnyx gateway"""

    fteu: Optional[bool] = None
    """Indicates whether this is a Free-To-End-User (FTEU) short code message"""

    mcc: Optional[str] = None
    """Mobile country code.

    Only available for certain products, such as Global Outbound-Only from
    Alphanumeric Sender ID
    """

    message_type: Optional[Literal["SMS", "MMS", "RCS"]] = None
    """Describes the Messaging service used to send the message.

    Available services are: Short Message Service (SMS), Multimedia Messaging
    Service (MMS), and Rich Communication Services (RCS)
    """

    mnc: Optional[str] = None
    """Mobile network code.

    Only available for certain products, such as Global Outbound-Only from
    Alphanumeric Sender ID
    """

    on_net: Optional[bool] = None
    """Indicates whether both sender and recipient numbers are Telnyx-managed"""

    parts: Optional[int] = None
    """Number of message parts.

    The message is broken down in multiple parts when its length surpasses the limit
    of 160 characters
    """

    profile_id: Optional[str] = None
    """Unique identifier of the Messaging Profile used to send or receive the message"""

    profile_name: Optional[str] = None
    """Name of the Messaging Profile used to send or receive the message"""

    rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing cost"""

    sent_at: Optional[datetime] = None
    """Time when the message was sent"""

    source_country_code: Optional[str] = None
    """
    Two-letter representation of the country of the cli property using the ISO
    3166-1 alpha-2 format
    """

    status: Optional[
        Literal["gw_timeout", "delivered", "dlr_unconfirmed", "dlr_timeout", "received", "gw_reject", "failed"]
    ] = None
    """Final status of the message after the delivery attempt"""

    tags: Optional[str] = None
    """Comma-separated tags assigned to the Telnyx number associated with the message"""

    updated_at: Optional[datetime] = None
    """Message updated time"""

    user_id: Optional[str] = None
    """Identifier of the Telnyx account who owns the message"""

    uuid: Optional[str] = None
    """Unique identifier of the message"""


class DataConferenceDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Conference id"""

    call_leg_id: Optional[str] = None
    """Telnyx UUID that identifies the conference call leg"""

    call_sec: Optional[int] = None
    """Duration of the conference call in seconds"""

    call_session_id: Optional[str] = None
    """Telnyx UUID that identifies with conference call session"""

    connection_id: Optional[str] = None
    """Connection id"""

    ended_at: Optional[datetime] = None
    """Conference end time"""

    expires_at: Optional[datetime] = None
    """Conference expiry time"""

    is_telnyx_billable: Optional[bool] = None
    """Indicates whether Telnyx billing charges might be applicable"""

    name: Optional[str] = None
    """Conference name"""

    participant_call_sec: Optional[int] = None
    """Sum of the conference call duration for all participants in seconds"""

    participant_count: Optional[int] = None
    """Number of participants that joined the conference call"""

    region: Optional[str] = None
    """Region where the conference is hosted"""

    started_at: Optional[datetime] = None
    """Conference start time"""

    user_id: Optional[str] = None
    """User id"""


class DataConferenceParticipantDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Participant id"""

    billed_sec: Optional[int] = None
    """Duration of the conference call for billing purposes"""

    call_leg_id: Optional[str] = None
    """Telnyx UUID that identifies the conference call leg"""

    call_sec: Optional[int] = None
    """Duration of the conference call in seconds"""

    call_session_id: Optional[str] = None
    """Telnyx UUID that identifies with conference call session"""

    conference_id: Optional[str] = None
    """Conference id"""

    cost: Optional[str] = None
    """Currency amount for Telnyx billing cost"""

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing cost
    """

    destination_number: Optional[str] = None
    """Number called by the participant to join the conference"""

    is_telnyx_billable: Optional[bool] = None
    """Indicates whether Telnyx billing charges might be applicable"""

    joined_at: Optional[datetime] = None
    """Participant join time"""

    left_at: Optional[datetime] = None
    """Participant leave time"""

    originating_number: Optional[str] = None
    """Participant origin number used in the conference call"""

    rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing cost"""

    rate_measured_in: Optional[str] = None
    """Billing unit used to calculate the Telnyx billing cost"""

    user_id: Optional[str] = None
    """User id"""


class DataAmdDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Feature invocation id"""

    billing_group_id: Optional[str] = None
    """Billing Group id"""

    billing_group_name: Optional[str] = None
    """Name of the Billing Group specified in billing_group_id"""

    call_leg_id: Optional[str] = None
    """Telnyx UUID that identifies the related call leg"""

    call_session_id: Optional[str] = None
    """Telnyx UUID that identifies the related call session"""

    connection_id: Optional[str] = None
    """Connection id"""

    connection_name: Optional[str] = None
    """Connection name"""

    cost: Optional[str] = None
    """Currency amount for Telnyx billing cost"""

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing cost
    """

    feature: Optional[Literal["PREMIUM"]] = None
    """Feature name"""

    invoked_at: Optional[datetime] = None
    """Feature invocation time"""

    is_telnyx_billable: Optional[bool] = None
    """Indicates whether Telnyx billing charges might be applicable"""

    rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing cost"""

    rate_measured_in: Optional[str] = None
    """Billing unit used to calculate the Telnyx billing cost"""

    tags: Optional[str] = None
    """User-provided tags"""


class DataVerifyDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Unique ID of the verification"""

    created_at: Optional[datetime] = None

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing
    costs
    """

    delivery_status: Optional[str] = None

    destination_phone_number: Optional[str] = None
    """E.164 formatted phone number"""

    rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing costs"""

    rate_measured_in: Optional[str] = None
    """Billing unit used to calculate the Telnyx billing costs"""

    updated_at: Optional[datetime] = None

    verification_status: Optional[str] = None

    verify_channel_id: Optional[str] = None

    verify_channel_type: Optional[Literal["sms", "psd2", "call", "flashcall"]] = None
    """
    Depending on the type of verification, the `verify_channel_id` points to one of
    the following channel ids;

    ---

    | verify_channel_type | verify_channel_id |
    | ------------------- | ----------------- |
    | sms, psd2           | messaging_id      |
    | call, flashcall     | call_control_id   |

    ---
    """

    verify_profile_id: Optional[str] = None

    verify_usage_fee: Optional[str] = None
    """Currency amount for Verify Usage Fee"""


class DataSimCardUsageDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Unique identifier for this SIM Card Usage"""

    closed_at: Optional[datetime] = None
    """Event close time"""

    created_at: Optional[datetime] = None
    """Event creation time"""

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing cost
    """

    data_cost: Optional[float] = None
    """Data cost"""

    data_rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing cost"""

    data_unit: Optional[str] = None
    """Unit of wireless link consumption"""

    downlink_data: Optional[float] = None
    """Number of megabytes downloaded"""

    imsi: Optional[str] = None
    """International Mobile Subscriber Identity"""

    ip_address: Optional[str] = None
    """Ip address that generated the event"""

    mcc: Optional[str] = None
    """Mobile country code"""

    mnc: Optional[str] = None
    """Mobile network code"""

    phone_number: Optional[str] = None
    """Telephone number associated to SIM card"""

    sim_card_id: Optional[str] = None
    """Unique identifier for SIM card"""

    sim_card_tags: Optional[str] = None
    """User-provided tags"""

    sim_group_id: Optional[str] = None
    """Unique identifier for SIM group"""

    sim_group_name: Optional[str] = None
    """Sim group name for sim card"""

    uplink_data: Optional[float] = None
    """Number of megabytes uploaded"""


class DataMediaStorageDetailRecord(BaseModel):
    record_type: str

    id: Optional[str] = None
    """Unique identifier for the Media Storage Event"""

    action_type: Optional[str] = None
    """Type of action performed against the Media Storage API"""

    asset_id: Optional[str] = None
    """Asset id"""

    cost: Optional[str] = None
    """Currency amount for Telnyx billing cost"""

    created_at: Optional[datetime] = None
    """Event creation time"""

    currency: Optional[str] = None
    """
    Telnyx account currency used to describe monetary values, including billing cost
    """

    link_channel_id: Optional[str] = None
    """Link channel id"""

    link_channel_type: Optional[str] = None
    """Link channel type"""

    org_id: Optional[str] = None
    """Organization owner id"""

    rate: Optional[str] = None
    """Currency amount per billing unit used to calculate the Telnyx billing cost"""

    rate_measured_in: Optional[str] = None
    """Billing unit used to calculate the Telnyx billing cost"""

    status: Optional[str] = None
    """Request status"""

    user_id: Optional[str] = None
    """User id"""

    webhook_id: Optional[str] = None
    """Webhook id"""


Data: TypeAlias = Annotated[
    Union[
        DataMessageDetailRecord,
        DataConferenceDetailRecord,
        DataConferenceParticipantDetailRecord,
        DataAmdDetailRecord,
        DataVerifyDetailRecord,
        DataSimCardUsageDetailRecord,
        DataMediaStorageDetailRecord,
    ],
    PropertyInfo(discriminator="record_type"),
]


class Meta(BaseModel):
    page_number: Optional[int] = None

    page_size: Optional[int] = None

    total_pages: Optional[int] = None

    total_results: Optional[int] = None


class DetailRecordListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .telnyx_downstream_campaign import TelnyxDownstreamCampaign

__all__ = ["PartnerCampaignListResponse"]


class PartnerCampaignListResponse(BaseModel):
    page: Optional[int] = None

    records: Optional[List[TelnyxDownstreamCampaign]] = None

    total_records: Optional[int] = FieldInfo(alias="totalRecords", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SubNumberOrderRetrieveParams", "Filter"]


class SubNumberOrderRetrieveParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[include_phone_numbers]
    """


class Filter(TypedDict, total=False):
    include_phone_numbers: bool
    """Include the first 50 phone number objects in the results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MigrationSourceParams", "ProviderAuth"]


class ProviderAuth(BaseModel):
    access_key: Optional[str] = None
    """AWS Access Key. For Telnyx-to-Telnyx migrations, use your Telnyx API key here."""

    secret_access_key: Optional[str] = None
    """AWS Secret Access Key.

    For Telnyx-to-Telnyx migrations, use your Telnyx API key here as well.
    """


class MigrationSourceParams(BaseModel):
    bucket_name: str
    """Bucket name to migrate the data from."""

    provider: Literal["aws", "telnyx"]
    """Cloud provider from which to migrate data.

    Use 'telnyx' if you want to migrate data from one Telnyx bucket to another.
    """

    provider_auth: ProviderAuth

    id: Optional[str] = None
    """Unique identifier for the data migration source."""

    source_region: Optional[str] = None
    """
    For intra-Telnyx buckets migration, specify the source bucket region in this
    field.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TestListParams", "Page"]


class TestListParams(TypedDict, total=False):
    destination: str
    """Filter tests by destination (phone number, webhook URL, etc.)"""

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """

    telnyx_conversation_channel: str
    """Filter tests by communication channel (e.g., 'web_chat', 'sms')"""

    test_suite: str
    """Filter tests by test suite name"""


class Page(TypedDict, total=False):
    number: int
    """Page number to retrieve (1-based indexing)"""

    size: int
    """Number of tests to return per page (1-100)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ExternalConnectionCreateParams", "Outbound", "Inbound"]


class ExternalConnectionCreateParams(TypedDict, total=False):
    external_sip_connection: Required[Literal["zoom"]]
    """The service that will be consuming this connection."""

    outbound: Required[Outbound]

    active: bool
    """Specifies whether the connection can be used."""

    inbound: Inbound

    tags: SequenceNotStr[str]
    """Tags associated with the connection."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: str
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""


class Outbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the number of concurrent outbound calls to phone
    numbers associated with this connection.
    """

    outbound_voice_profile_id: str
    """Identifies the associated outbound voice profile."""


class Inbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the number of concurrent inbound calls to phone
    numbers associated with this connection.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypedDict

__all__ = ["InsightCreateParams"]


class InsightCreateParams(TypedDict, total=False):
    instructions: Required[str]

    name: Required[str]

    json_schema: Union[str, object]
    """If specified, the output will follow the JSON schema."""

    webhook: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SimCardGroupListResponse,
    SimCardGroupCreateResponse,
    SimCardGroupDeleteResponse,
    SimCardGroupUpdateResponse,
    SimCardGroupRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSimCardGroups:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.create(
            name="My Test Group",
        )
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.create(
            name="My Test Group",
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
        )
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.sim_card_groups.with_raw_response.create(
            name="My Test Group",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = response.parse()
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.sim_card_groups.with_streaming_response.create(
            name="My Test Group",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = response.parse()
            assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            include_iccids=True,
        )
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sim_card_groups.with_raw_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = response.parse()
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sim_card_groups.with_streaming_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = response.parse()
            assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
            name="My Test Group",
        )
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.sim_card_groups.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = response.parse()
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.sim_card_groups.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = response.parse()
            assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.list()
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.list(
            filter_name="My Test Group",
            filter_private_wireless_gateway_id="7606c6d3-ff7c-49c1-943d-68879e9d584d",
            filter_wireless_blocklist_id="0f3f490e-c4d3-4cf5-838a-9970f10ee259",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sim_card_groups.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = response.parse()
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sim_card_groups.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = response.parse()
            assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        sim_card_group = client.sim_card_groups.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.sim_card_groups.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = response.parse()
        assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.sim_card_groups.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = response.parse()
            assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.with_raw_response.delete(
                "",
            )


class TestAsyncSimCardGroups:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.create(
            name="My Test Group",
        )
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.create(
            name="My Test Group",
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
        )
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.with_raw_response.create(
            name="My Test Group",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = await response.parse()
        assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.with_streaming_response.create(
            name="My Test Group",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = await response.parse()
            assert_matches_type(SimCardGroupCreateResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            include_iccids=True,
        )
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.with_raw_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = await response.parse()
        assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.with_streaming_response.retrieve(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = await response.parse()
            assert_matches_type(SimCardGroupRetrieveResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            data_limit={
                "amount": "2048.1",
                "unit": "MB",
            },
            name="My Test Group",
        )
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = await response.parse()
        assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = await response.parse()
            assert_matches_type(SimCardGroupUpdateResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.list()
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.list(
            filter_name="My Test Group",
            filter_private_wireless_gateway_id="7606c6d3-ff7c-49c1-943d-68879e9d584d",
            filter_wireless_blocklist_id="0f3f490e-c4d3-4cf5-838a-9970f10ee259",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = await response.parse()
        assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = await response.parse()
            assert_matches_type(SimCardGroupListResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        sim_card_group = await async_client.sim_card_groups.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sim_card_group = await response.parse()
        assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sim_card_group = await response.parse()
            assert_matches_type(SimCardGroupDeleteResponse, sim_card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import short_code_list_params, short_code_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.short_code_list_response import ShortCodeListResponse
from ..types.short_code_update_response import ShortCodeUpdateResponse
from ..types.short_code_retrieve_response import ShortCodeRetrieveResponse

__all__ = ["ShortCodesResource", "AsyncShortCodesResource"]


class ShortCodesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ShortCodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ShortCodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ShortCodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ShortCodesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeRetrieveResponse:
        """
        Retrieve a short code

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/short_codes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ShortCodeRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        messaging_profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeUpdateResponse:
        """Update the settings for a specific short code.

        To unbind a short code from a
        profile, set the `messaging_profile_id` to `null` or an empty string.

        Args:
          messaging_profile_id: Unique identifier for a messaging profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/short_codes/{id}",
            body=maybe_transform(
                {"messaging_profile_id": messaging_profile_id}, short_code_update_params.ShortCodeUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ShortCodeUpdateResponse,
        )

    def list(
        self,
        *,
        filter: short_code_list_params.Filter | Omit = omit,
        page: short_code_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeListResponse:
        """
        List short codes

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[messaging_profile_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/short_codes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    short_code_list_params.ShortCodeListParams,
                ),
            ),
            cast_to=ShortCodeListResponse,
        )


class AsyncShortCodesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncShortCodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncShortCodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncShortCodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncShortCodesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeRetrieveResponse:
        """
        Retrieve a short code

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/short_codes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ShortCodeRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        messaging_profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeUpdateResponse:
        """Update the settings for a specific short code.

        To unbind a short code from a
        profile, set the `messaging_profile_id` to `null` or an empty string.

        Args:
          messaging_profile_id: Unique identifier for a messaging profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/short_codes/{id}",
            body=await async_maybe_transform(
                {"messaging_profile_id": messaging_profile_id}, short_code_update_params.ShortCodeUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ShortCodeUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: short_code_list_params.Filter | Omit = omit,
        page: short_code_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ShortCodeListResponse:
        """
        List short codes

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[messaging_profile_id]

          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/short_codes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    short_code_list_params.ShortCodeListParams,
                ),
            ),
            cast_to=ShortCodeListResponse,
        )


class ShortCodesResourceWithRawResponse:
    def __init__(self, short_codes: ShortCodesResource) -> None:
        self._short_codes = short_codes

        self.retrieve = to_raw_response_wrapper(
            short_codes.retrieve,
        )
        self.update = to_raw_response_wrapper(
            short_codes.update,
        )
        self.list = to_raw_response_wrapper(
            short_codes.list,
        )


class AsyncShortCodesResourceWithRawResponse:
    def __init__(self, short_codes: AsyncShortCodesResource) -> None:
        self._short_codes = short_codes

        self.retrieve = async_to_raw_response_wrapper(
            short_codes.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            short_codes.update,
        )
        self.list = async_to_raw_response_wrapper(
            short_codes.list,
        )


class ShortCodesResourceWithStreamingResponse:
    def __init__(self, short_codes: ShortCodesResource) -> None:
        self._short_codes = short_codes

        self.retrieve = to_streamed_response_wrapper(
            short_codes.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            short_codes.update,
        )
        self.list = to_streamed_response_wrapper(
            short_codes.list,
        )


class AsyncShortCodesResourceWithStreamingResponse:
    def __init__(self, short_codes: AsyncShortCodesResource) -> None:
        self._short_codes = short_codes

        self.retrieve = async_to_streamed_response_wrapper(
            short_codes.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            short_codes.update,
        )
        self.list = async_to_streamed_response_wrapper(
            short_codes.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.addresses import (
    ActionValidateResponse,
    ActionAcceptSuggestionsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_accept_suggestions(self, client: Telnyx) -> None:
        action = client.addresses.actions.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_accept_suggestions_with_all_params(self, client: Telnyx) -> None:
        action = client.addresses.actions.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body_id="id",
        )
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_accept_suggestions(self, client: Telnyx) -> None:
        response = client.addresses.actions.with_raw_response.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_accept_suggestions(self, client: Telnyx) -> None:
        with client.addresses.actions.with_streaming_response.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_accept_suggestions(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            client.addresses.actions.with_raw_response.accept_suggestions(
                path_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_validate(self, client: Telnyx) -> None:
        action = client.addresses.actions.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        )
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_validate_with_all_params(self, client: Telnyx) -> None:
        action = client.addresses.actions.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
            administrative_area="TX",
            extended_address="14th Floor",
            locality="Austin",
        )
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_validate(self, client: Telnyx) -> None:
        response = client.addresses.actions.with_raw_response.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_validate(self, client: Telnyx) -> None:
        with client.addresses.actions.with_streaming_response.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionValidateResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncActions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_accept_suggestions(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.addresses.actions.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_accept_suggestions_with_all_params(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.addresses.actions.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body_id="id",
        )
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_accept_suggestions(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.addresses.actions.with_raw_response.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_accept_suggestions(self, async_client: AsyncTelnyx) -> None:
        async with async_client.addresses.actions.with_streaming_response.accept_suggestions(
            path_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionAcceptSuggestionsResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_accept_suggestions(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_id` but received ''"):
            await async_client.addresses.actions.with_raw_response.accept_suggestions(
                path_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_validate(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.addresses.actions.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        )
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_validate_with_all_params(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.addresses.actions.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
            administrative_area="TX",
            extended_address="14th Floor",
            locality="Austin",
        )
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_validate(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.addresses.actions.with_raw_response.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionValidateResponse, action, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_validate(self, async_client: AsyncTelnyx) -> None:
        async with async_client.addresses.actions.with_streaming_response.validate(
            country_code="US",
            postal_code="78701",
            street_address="600 Congress Avenue",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionValidateResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.storage.buckets import ssl_certificate_create_params
from ....types.storage.buckets.ssl_certificate_create_response import SslCertificateCreateResponse
from ....types.storage.buckets.ssl_certificate_delete_response import SslCertificateDeleteResponse
from ....types.storage.buckets.ssl_certificate_retrieve_response import SslCertificateRetrieveResponse

__all__ = ["SslCertificateResource", "AsyncSslCertificateResource"]


class SslCertificateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SslCertificateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SslCertificateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SslCertificateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SslCertificateResourceWithStreamingResponse(self)

    def create(
        self,
        bucket_name: str,
        *,
        certificate: FileTypes | Omit = omit,
        private_key: FileTypes | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateCreateResponse:
        """
        Uploads an SSL certificate and its matching secret so that you can use Telnyx’s
        storage as your CDN.

        Args:
          certificate: The SSL certificate file

          private_key: The private key file

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        body = deepcopy_minimal(
            {
                "certificate": certificate,
                "private_key": private_key,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["certificate"], ["private_key"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._put(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            body=maybe_transform(body, ssl_certificate_create_params.SslCertificateCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateCreateResponse,
        )

    def retrieve(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateRetrieveResponse:
        """
        Returns the stored certificate detail of a bucket, if applicable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateRetrieveResponse,
        )

    def delete(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateDeleteResponse:
        """
        Deletes an SSL certificate and its matching secret.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._delete(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateDeleteResponse,
        )


class AsyncSslCertificateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSslCertificateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSslCertificateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSslCertificateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSslCertificateResourceWithStreamingResponse(self)

    async def create(
        self,
        bucket_name: str,
        *,
        certificate: FileTypes | Omit = omit,
        private_key: FileTypes | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateCreateResponse:
        """
        Uploads an SSL certificate and its matching secret so that you can use Telnyx’s
        storage as your CDN.

        Args:
          certificate: The SSL certificate file

          private_key: The private key file

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        body = deepcopy_minimal(
            {
                "certificate": certificate,
                "private_key": private_key,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["certificate"], ["private_key"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._put(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            body=await async_maybe_transform(body, ssl_certificate_create_params.SslCertificateCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateCreateResponse,
        )

    async def retrieve(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateRetrieveResponse:
        """
        Returns the stored certificate detail of a bucket, if applicable.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateRetrieveResponse,
        )

    async def delete(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SslCertificateDeleteResponse:
        """
        Deletes an SSL certificate and its matching secret.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._delete(
            f"/storage/buckets/{bucket_name}/ssl_certificate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SslCertificateDeleteResponse,
        )


class SslCertificateResourceWithRawResponse:
    def __init__(self, ssl_certificate: SslCertificateResource) -> None:
        self._ssl_certificate = ssl_certificate

        self.create = to_raw_response_wrapper(
            ssl_certificate.create,
        )
        self.retrieve = to_raw_response_wrapper(
            ssl_certificate.retrieve,
        )
        self.delete = to_raw_response_wrapper(
            ssl_certificate.delete,
        )


class AsyncSslCertificateResourceWithRawResponse:
    def __init__(self, ssl_certificate: AsyncSslCertificateResource) -> None:
        self._ssl_certificate = ssl_certificate

        self.create = async_to_raw_response_wrapper(
            ssl_certificate.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            ssl_certificate.retrieve,
        )
        self.delete = async_to_raw_response_wrapper(
            ssl_certificate.delete,
        )


class SslCertificateResourceWithStreamingResponse:
    def __init__(self, ssl_certificate: SslCertificateResource) -> None:
        self._ssl_certificate = ssl_certificate

        self.create = to_streamed_response_wrapper(
            ssl_certificate.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ssl_certificate.retrieve,
        )
        self.delete = to_streamed_response_wrapper(
            ssl_certificate.delete,
        )


class AsyncSslCertificateResourceWithStreamingResponse:
    def __init__(self, ssl_certificate: AsyncSslCertificateResource) -> None:
        self._ssl_certificate = ssl_certificate

        self.create = async_to_streamed_response_wrapper(
            ssl_certificate.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ssl_certificate.retrieve,
        )
        self.delete = async_to_streamed_response_wrapper(
            ssl_certificate.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["PortoutListParams", "Filter", "FilterInsertedAt", "FilterPortedOutAt", "Page"]


class PortoutListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[carrier_name], filter[country_code], filter[country_code_in],
    filter[foc_date], filter[inserted_at], filter[phone_number], filter[pon],
    filter[ported_out_at], filter[spid], filter[status], filter[status_in],
    filter[support_key]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class FilterInsertedAt(TypedDict, total=False):
    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by inserted_at date greater than or equal."""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by inserted_at date less than or equal."""


class FilterPortedOutAt(TypedDict, total=False):
    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by ported_out_at date greater than or equal."""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by ported_out_at date less than or equal."""


class Filter(TypedDict, total=False):
    carrier_name: str
    """Filter by new carrier name."""

    country_code: str
    """Filter by 2-letter country code"""

    country_code_in: SequenceNotStr[str]
    """Filter by a list of 2-letter country codes"""

    foc_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by foc_date. Matches all portouts with the same date"""

    inserted_at: FilterInsertedAt
    """Filter by inserted_at date range using nested operations"""

    phone_number: str
    """Filter by a phone number on the portout.

    Matches all portouts with the phone number
    """

    pon: str
    """Filter by Port Order Number (PON)."""

    ported_out_at: FilterPortedOutAt
    """Filter by ported_out_at date range using nested operations"""

    spid: str
    """Filter by new carrier spid."""

    status: Literal["pending", "authorized", "ported", "rejected", "rejected-pending", "canceled"]
    """Filter by portout status."""

    status_in: List[Literal["pending", "authorized", "ported", "rejected", "rejected-pending", "canceled"]]
    """Filter by a list of portout statuses"""

    support_key: str
    """Filter by the portout's support_key"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .rcs_suggestion import RcsSuggestion
from .rcs_content_info import RcsContentInfo

__all__ = ["RcsCardContent", "Media"]


class Media(BaseModel):
    content_info: Optional[RcsContentInfo] = None

    height: Optional[Literal["HEIGHT_UNSPECIFIED", "SHORT", "MEDIUM", "TALL"]] = None
    """The height of the media within a rich card with a vertical layout.

    For a standalone card with horizontal layout, height is not customizable, and
    this field is ignored.
    """


class RcsCardContent(BaseModel):
    description: Optional[str] = None
    """Description of the card (at most 2000 characters)"""

    media: Optional[Media] = None
    """A media file within a rich card."""

    suggestions: Optional[List[RcsSuggestion]] = None
    """List of suggestions to include in the card. Maximum 10 suggestions."""

    title: Optional[str] = None
    """Title of the card (at most 200 characters)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PortoutUpdateStatusParams"]


class PortoutUpdateStatusParams(TypedDict, total=False):
    id: Required[str]

    reason: Required[str]
    """Provide a reason if rejecting the port out request"""

    host_messaging: bool
    """
    Indicates whether messaging services should be maintained with Telnyx after the
    port out completes
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .url import URL
from .volume import Volume
from ...._models import BaseModel
from .tf_phone_number import TfPhoneNumber
from .use_case_categories import UseCaseCategories
from .tf_verification_status import TfVerificationStatus
from .toll_free_verification_entity_type import TollFreeVerificationEntityType

__all__ = ["VerificationRequestStatus"]


class VerificationRequestStatus(BaseModel):
    id: str

    additional_information: str = FieldInfo(alias="additionalInformation")

    business_addr1: str = FieldInfo(alias="businessAddr1")

    business_city: str = FieldInfo(alias="businessCity")

    business_contact_email: str = FieldInfo(alias="businessContactEmail")

    business_contact_first_name: str = FieldInfo(alias="businessContactFirstName")

    business_contact_last_name: str = FieldInfo(alias="businessContactLastName")

    business_contact_phone: str = FieldInfo(alias="businessContactPhone")

    business_name: str = FieldInfo(alias="businessName")

    business_state: str = FieldInfo(alias="businessState")

    business_zip: str = FieldInfo(alias="businessZip")

    corporate_website: str = FieldInfo(alias="corporateWebsite")

    isv_reseller: str = FieldInfo(alias="isvReseller")

    message_volume: Volume = FieldInfo(alias="messageVolume")
    """Message Volume Enums"""

    opt_in_workflow: str = FieldInfo(alias="optInWorkflow")

    opt_in_workflow_image_urls: List[URL] = FieldInfo(alias="optInWorkflowImageURLs")

    phone_numbers: List[TfPhoneNumber] = FieldInfo(alias="phoneNumbers")

    production_message_content: str = FieldInfo(alias="productionMessageContent")

    use_case: UseCaseCategories = FieldInfo(alias="useCase")
    """Tollfree usecase categories"""

    use_case_summary: str = FieldInfo(alias="useCaseSummary")

    verification_status: TfVerificationStatus = FieldInfo(alias="verificationStatus")
    """Tollfree verification status"""

    age_gated_content: Optional[bool] = FieldInfo(alias="ageGatedContent", default=None)

    business_addr2: Optional[str] = FieldInfo(alias="businessAddr2", default=None)

    business_registration_country: Optional[str] = FieldInfo(alias="businessRegistrationCountry", default=None)

    business_registration_number: Optional[str] = FieldInfo(alias="businessRegistrationNumber", default=None)

    business_registration_type: Optional[str] = FieldInfo(alias="businessRegistrationType", default=None)

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    doing_business_as: Optional[str] = FieldInfo(alias="doingBusinessAs", default=None)

    entity_type: Optional[TollFreeVerificationEntityType] = FieldInfo(alias="entityType", default=None)
    """Business entity classification"""

    help_message_response: Optional[str] = FieldInfo(alias="helpMessageResponse", default=None)

    opt_in_confirmation_response: Optional[str] = FieldInfo(alias="optInConfirmationResponse", default=None)

    opt_in_keywords: Optional[str] = FieldInfo(alias="optInKeywords", default=None)

    privacy_policy_url: Optional[str] = FieldInfo(alias="privacyPolicyURL", default=None)

    reason: Optional[str] = None

    terms_and_condition_url: Optional[str] = FieldInfo(alias="termsAndConditionURL", default=None)

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)

    webhook_url: Optional[str] = FieldInfo(alias="webhookUrl", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .stt_detail_report_response import SttDetailReportResponse

__all__ = ["SpeechToTextListResponse"]


class SpeechToTextListResponse(BaseModel):
    data: Optional[List[SttDetailReportResponse]] = None
